@@ -6,17 +6,17 @@ import numpy as np
 from spacy import util
 from spacy.tokens import Doc
 
-from .classy_skeleton import (
-    classyExternal,
-    classySkeleton,
-    classySkeletonFewShot,
-    classySkeletonFewShotMultiLabel,
-)
+from .classy_skeleton import ClassyExternal, ClassySkeleton, ClassySkeletonFewShot
 
 
-class classySpacy:
+class ClassySpacy:
     def sentence_pipe(self, doc: Doc):
-        sent_docs = [sent.as_doc() for sent in doc.sents]
+        if doc.has_extension("trf_data"):
+            disable = [comp[0] for comp in self.nlp.components if comp[0] != "transformer"]
+            texts = [sent.text for sent in doc.sents]
+            sent_docs = self.nlp.pipe(texts, disable=disable)
+        else:
+            sent_docs = [sent.as_doc() for sent in doc.sents]
         inferred_sent_docs = self.pipe(iter(sent_docs), include_sent=False)
         for sent_doc, sent in zip(inferred_sent_docs, doc.sents):
             sent._.cats = sent_doc._.cats
@@ -67,7 +67,7 @@ class classySpacy:
                 yield doc
 
 
-class classySpacyInternal(classySpacy):
+class ClassySpacyInternal(ClassySpacy):
     def get_embeddings(self, docs: Union[List[Doc], List[str]]) -> List[float]:
         """Retrieve embeddings from text.
         Overwrites function from the classySkeleton that is used to get embeddings for training data to fetch internal
@@ -92,64 +92,52 @@ class classySpacyInternal(classySpacy):
             if doc.has_vector:
                 embeddings.append(doc.vector)
             else:
-                raise NotImplementedError(
-                    "internal spacy embeddings need to be derived from md/lg spacy models not from sm/trf models."
-                )
+                if doc.has_extension("trf_data"):
+                    embeddings.append(doc._.trf_data.model_output.pooler_output[0])
+                else:
+                    raise NotImplementedError(
+                        "internal spacy embeddings need to be derived from md/lg/trf spacy models not from sm models."
+                    )
 
         return np.array(embeddings)
 
 
-class classySpacyInternalFewShot(classySpacyInternal, classySkeletonFewShot):
+class ClassySpacyInternalFewShot(ClassySpacyInternal, ClassySkeletonFewShot):
     def __init__(self, *args, **kwargs):
-        classySkeletonFewShot.__init__(self, *args, **kwargs)
+        ClassySkeletonFewShot.__init__(self, *args, **kwargs)
 
 
-class classySpacyInternalFewShotMultiLabel(classySpacyInternal, classySkeletonFewShotMultiLabel):
-    def __init__(self, *args, **kwargs):
-        classySkeletonFewShotMultiLabel.__init__(self, *args, **kwargs)
-
-
-class classySpacyExternalFewShot(classySpacy, classyExternal, classySkeletonFewShot):
+class ClassySpacyExternalFewShot(ClassySpacy, ClassyExternal, ClassySkeletonFewShot):
     def __init__(
         self,
-        model: str = "sentence-transformers/paraphrase-multilingual-MiniLM-L12-v2",
+        model: str = None,
         device: str = "cpu",
         *args,
         **kwargs,
     ):
+        if model is None:
+            model = "sentence-transformers/paraphrase-multilingual-MiniLM-L12-v2"
         self.model = model
         self.device = device
         self.set_embedding_model()
-        classySkeletonFewShot.__init__(self, *args, **kwargs)
+        ClassySkeletonFewShot.__init__(self, *args, **kwargs)
 
 
-class classySpacyExternalFewShotMultiLabel(classySpacy, classyExternal, classySkeletonFewShotMultiLabel):
+class ClassySpacyExternalZeroShot(ClassySpacy, ClassySkeleton):
     def __init__(
         self,
-        model: str = "sentence-transformers/paraphrase-multilingual-MiniLM-L12-v2",
-        device: str = "cpu",
-        *args,
-        **kwargs,
-    ):
-        self.model = model
-        self.device = device
-        self.set_embedding_model()
-        classySkeletonFewShotMultiLabel.__init__(self, *args, **kwargs)
-
-
-class classySpacyExternalZeroShot(classySpacy, classySkeleton):
-    def __init__(
-        self,
-        model: str = "typeform/distilbert-base-uncased-mnli",
+        model: str = None,
         device: str = "cpu",
         multi_label: bool = False,
         *args,
         **kwargs,
     ):
+        if model is None:
+            model = "typeform/distilbert-base-uncased-mnli"
         self.model = model
         self.device = device
         self.multi_label = multi_label
-        classySkeleton.__init__(self, *args, **kwargs)
+        ClassySkeleton.__init__(self, *args, **kwargs)
 
     def set_classification_model(self, model: str = None, device: str = None):
         """set the embedding model based on a sentencetransformer model or path
